public class SharingPlug : Pantheon.Switchboard.Plug {

    // Set hostname
    void set_host (Gtk.Entry hostname_entry) {
        string current_host = Environment.get_host_name ();

        string new_host = hostname_entry.get_text ();

        var file4 = File.new_for_path ("/etc/hosts");

        var dis4 = new DataInputStream (file4.read ());
        string line4;
        string conf = "";
        while ((line4 = dis4.read_line (null)) != null) {
            conf = conf + line4 + "\n";
        }

        conf = conf.replace(current_host, new_host);

        File f = File.new_for_commandline_arg(Environment.get_home_dir () + "/.hosts.tmp");
        FileOutputStream fo_stream = null;
        if(f.query_exists(null))
            f.delete(null);
        fo_stream = f.create(FileCreateFlags.REPLACE_DESTINATION, null);

        fo_stream.write(conf.data, null);
        Process.spawn_command_line_sync("gksudo mv " + Environment.get_home_dir () + "/.hosts.tmp /etc/hosts");

        File f2 = File.new_for_commandline_arg(Environment.get_home_dir () + "/.hostname.tmp");
        FileOutputStream fo_stream2 = null;
        if(f2.query_exists(null))
            f2.delete(null);
        fo_stream2 = f2.create(FileCreateFlags.REPLACE_DESTINATION, null);

        fo_stream2.write(new_host.data, null);
        Process.spawn_command_line_sync("gksudo mv " + Environment.get_home_dir () + "/.hostname.tmp /etc/hostname");
        Process.spawn_command_line_sync("gksudo hostname " + new_host);
    }

    public SharingPlug () {

        // Containers
        var main = new Gtk.HBox (true, 2);
        var left = new Gtk.VBox (false, 2);
        var right = new Gtk.VBox (true, 3);
        var buttons = new Gtk.HBox (true, 2);
        var hostname = new Gtk.HBox (false, 2);

        // File locations
        var samba = File.new_for_path ("/etc/init.d/smbd");
        var ssh = File.new_for_path ("/usr/sbin/sshd");
        var vnc = File.new_for_path ("/usr/bin/x11vnc");

        // Hostname setting
        var hostname_entry = new Gtk.Entry ();
        var hostname_label = new Gtk.Label ("Computer's name: ");

        hostname_entry.activate.connect (set_host);

        // Toggles
        var samba_toggle = new Gtk.CheckButton.with_mnemonic ("File Sharing");
        int samba_state = 2;

        var ssh_toggle = new Gtk.CheckButton.with_mnemonic ("Remote Login");
        int ssh_state = 2;

        var vnc_toggle = new Gtk.CheckButton.with_mnemonic ("Screen Sharing");
        int vnc_state = 2;

        samba_toggle.toggled.connect (() => {
            if (samba_state == 1) {
                Process.spawn_command_line_sync("gksudo 'apt-get remove -y samba'");
                if (!samba.query_exists ()) {
                    samba_state = 0;
                } else {
                    samba_state = 2;
                    samba_toggle.set_active (true);
                    samba_state = 1;
                }
            } else if (samba_state == 0){
                Process.spawn_command_line_sync("apturl apt://samba");
                if (samba.query_exists ()) {
                    samba_state = 1;
                } else {
                    samba_state = 2;
                    samba_toggle.set_active (false);
                    samba_state = 0;
                }
            }
        });

        ssh_toggle.toggled.connect (() => {
            if (ssh_state == 1) {
                Process.spawn_command_line_sync("gksudo 'apt-get remove -y openssh-server'");
                if (!ssh.query_exists ()) {
                    ssh_state = 0;
                } else {
                    ssh_state = 2;
                    ssh_toggle.set_active (true);
                    ssh_state = 1;
                }
            } else if (ssh_state == 0){
                Process.spawn_command_line_sync("apturl apt://openssh-server");
                if (ssh.query_exists ()) {
                    ssh_state = 1;
                } else {
                    ssh_state = 2;
                    ssh_toggle.set_active (false);
                    ssh_state = 0;
                }
            }
        });

        vnc_toggle.toggled.connect (() => {
            if (vnc_state == 1) {
                Process.spawn_command_line_sync("gksudo 'apt-get remove -y x11vnc'");
                if (!vnc.query_exists ()) {
                    vnc_state = 0;
                } else {
                    vnc_vnc = 2;
                    vnc_toggle.set_active (true);
                    vnc_state = 1;
                }
            } else if (vnc_state == 0){
                Process.spawn_command_line_sync("apturl apt://x11vnc");
                if (vnc.query_exists ()) {
                    vnc_state = 1;
                } else {
                    vnc_state = 2;
                    vnc_toggle.set_active (false);
                    vnc_state = 0;
                }
            }
        });

        // Shares view
        var view = new Gtk.TreeView ();
        var listmodel = new Gtk.ListStore (4, typeof (string), typeof (string),
                                          typeof (string), typeof (string));
        view.set_model (listmodel);

        view.insert_column_with_attributes (-1, "Shared folders", new Gtk.CellRendererText (), "text", 0);

        var cell = new Gtk.CellRendererText ();
        cell.set ("foreground_set", true);

        Gtk.TreeIter iter;

        var file = File.new_for_path ("/etc/samba/smb.conf");

        var dis = new DataInputStream (file.read ());
        string line;
        while ((line = dis.read_line (null)) != null) {
            if (";" in line) {
            } else if ("#" in line) {
            } else if ("/var/spool/samba" in line) {
            } else if ("/var/lib/samba/printers" in line) {
            } else if ("path = " in line) {
                line = line.replace("path = ", "");
                line = Path.get_basename(line);
                listmodel.append (out iter);
                listmodel.set (iter, 0, line);
            }
        }

        // Buttons
        var add = new Gtk.Button.with_label("+");

        add.clicked.connect (() => {

            var file_chooser = new Gtk.FileChooserDialog ("Choose Folder", this,
                                          Gtk.FileChooserAction.SELECT_FOLDER,
                                          Gtk.Stock.CANCEL, Gtk.ResponseType.CANCEL,
                                          Gtk.Stock.OPEN, Gtk.ResponseType.ACCEPT);
            if (file_chooser.run () == Gtk.ResponseType.ACCEPT) {
                string folder = file_chooser.get_filename ();

                var file2 = File.new_for_path ("/etc/samba/smb.conf");

                var dis2 = new DataInputStream (file2.read ());
                string line2;
                string conf = "";
                while ((line2 = dis2.read_line (null)) != null) {
                    conf = conf + line2 + "\n";
                }

                conf = conf + "\n[" + Path.get_basename(folder) +  "]\n   comment = " + Path.get_basename(folder) + "\n   path = " + folder + "\n   browseable = yes\n   guest ok = yes";

                File f = File.new_for_commandline_arg(Environment.get_home_dir () + "/.smb.tmp");
                FileOutputStream fo_stream = null;

                if(f.query_exists(null))
                    f.delete(null);
                fo_stream = f.create(FileCreateFlags.REPLACE_DESTINATION, null);

                fo_stream.write(conf.data, null);
                Process.spawn_command_line_sync("gksudo mv " + Environment.get_home_dir () + "/.smb.tmp /etc/samba/smb.conf");
                Process.spawn_command_line_async("gksudo restart smbd");
                Process.spawn_command_line_async("gksudo restart nmbd");
            }
            file_chooser.destroy ();

        });

        var rm = new Gtk.Button.with_label("-");

        rm.clicked.connect (() => {

            var file_chooser = new Gtk.FileChooserDialog ("Choose Folder", this,
                                          Gtk.FileChooserAction.SELECT_FOLDER,
                                          Gtk.Stock.CANCEL, Gtk.ResponseType.CANCEL,
                                          Gtk.Stock.OPEN, Gtk.ResponseType.ACCEPT);
            if (file_chooser.run () == Gtk.ResponseType.ACCEPT) {
                string folder = file_chooser.get_filename ();

                var file3 = File.new_for_path ("/etc/samba/smb.conf");

                var dis3 = new DataInputStream (file3.read ());
                string line3;
                string conf = "";
                while ((line3 = dis3.read_line (null)) != null) {
                    conf = conf + line3 + "\n";
                }

                conf = conf.replace("   path = " + folder, "#   path =" + folder);

                File f = File.new_for_commandline_arg(Environment.get_home_dir () + "/.smb.tmp");
                FileOutputStream fo_stream = null;

                if(f.query_exists(null))
                    f.delete(null);
                fo_stream = f.create(FileCreateFlags.REPLACE_DESTINATION, null);

                fo_stream.write(conf.data, null);
                Process.spawn_command_line_sync("gksudo mv " + Environment.get_home_dir () + "/.smb.tmp /etc/samba/smb.conf");
                Process.spawn_command_line_async("gksudo restart smbd");
                Process.spawn_command_line_async("gksudo restart nmbd");
            }
            file_chooser.destroy ();
            
        });

        // Packing the containers
        buttons.pack_start (add, true, true, 0);
        buttons.pack_end (rm, true, true, 0);
        left.pack_start (view, true, true, 0);
        left.pack_end (buttons, false, false, 0);
        hostname.pack_start (hostname_label, false, false, 0);
        hostname.pack_end (hostname_entry, true, true, 0);
        right.pack_start (hostname, false, false, 0);
        right.pack_start (samba_toggle, false, false, 0);
        right.pack_end (ssh_toggle, false, false, 0);
        if (samba.query_exists ()) {
            main.pack_start (left, true, true, 0);
        }
        main.pack_end (right, true, true, 0);

        // Setting the states
        if (samba.query_exists ()) {
            samba_toggle.set_active (true);
            samba_state = 1;
        } else {
            samba_state = 0;
        }

        if (ssh.query_exists ()) {
            ssh_toggle.set_active (true);
            ssh_state = 1;
        } else {
            ssh_state = 0;
        }

        if (vnc.query_exists ()) {
            vnc_toggle.set_active (true);
            vnc_state = 1;
        } else {
            vnc_state = 0;
        }

        // Getting the current hostname
        hostname_entry.set_text (Environment.get_host_name());

        // Adding the main container
        this.add (main);
    }
}

public static int main (string[] args) {
    Gtk.init (ref args);
    var plug = new SharingPlug ();
    plug.register ("Sharing");
    plug.show_all ();
    Gtk.main ();
    return 0;
}
